/*
resizer-yuv.c

Copyright (C) Jan Panteltje  2002

resizer-yuv is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.
  
resizer-yuv is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
  
You should have received a copy of the GNU General Public License
along with GNU Make; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
*/

#include "resizer.h"

//#include "fftw.h"
#include <rfftw.h>

//fftw_real in[720], out[720];
fftw_real *in, *out;

rfftw_plan plan_luma, plan_chroma, plan_reverse_luma, plan_reverse_chroma;


char *strsave(char *s) /* save char array s somewhere */
{
char *p, *malloc();
if(p = malloc( strlen(s) +  1) ) strcpy(p, s);
return(p);
}


int main(int argc, char **argv)
{
int a, c, i, j;
char temp[4096];
double da, db, dc;
static int x, y, u, v, y1, r, g, b;
int idy;
uint8_t *py, *pu, *pv;
uint8_t *pyo, *puo, *pvo;
static int cr, cg, cb, cy, cu, cv;
int interlace_mode;
double aspect_ratio;
int length, ylength, ulength, vlength;
int new_length, new_ylength, new_ulength, new_vlength;
char *ptr;
FILE *outfile;
char *bptr;
int width, height, frame_rate1, frame_rate2;
int new_width, new_height;
int current_frame_nr;
char metadata[READSIZE];
int aspect_ratio1, aspect_ratio2;
int new_aspect_ratio1, new_aspect_ratio2;
char *running;                                             
char *token;
int reverse_uv_flag;
int fft_measure_flag;
int points;
double dh_scale_ratio;
double dcut_off_frequency;
int low_pass_only_flag;
char *inbuffer, *outbuffer;
double *dattenuation_factor;
double dfmax, dfstep;
double dfslope, dtslope, dslope_points;
double dslope_start, dslope_half, dslope_end;
int in_slope_flag;
double dangle;
int burst_mask;

/* identify */
fprintf(stderr, "\nPanteltje (c) resizer-yuv%s\n\n", VERSION);

/* get home directory */  
userinfo = getpwuid(getuid() );
home_dir = strsave(userinfo -> pw_dir);
user_name = strsave(userinfo -> pw_name);

/* set some defaults */
debug_flag = 0;

/* for stdin to stdout */
outfile = stdout;

reverse_uv_flag = 1;
fft_measure_flag = 0;
low_pass_only_flag = 0;

/* This signals to use width ratios to calculate the Niquist point */  
dcut_off_frequency = 0.0;

dfslope = 100000000.0; /* 100 MHz */

burst_mask = 0;
/* end defaults */

/* get command line options, these override the defaults */
while(1)
	{
	a = getopt(argc, argv, "b:df:hlmrs:x:y:");
	if(a == -1) break;
	switch(a)
		{
		case 'b':
			burst_mask = atoi(optarg);
			break;
        case 'd':
			debug_flag = 1;
			break;
		case 'f':
			dcut_off_frequency = 1000000.0 * atof(optarg);
			break;
		case 'h':
			fprintf(stderr,\
"Usage: resizer-yuv [-d] [-h] [-m] [-r] [-x width] [-y height].\n\n\
\t-b  pixels (int) mask out burst residue on left side of picture.\n\
\t-d  print debug messages (o.a. function calls and arguments).\n\
\t-f  frequency (in MHz) (float) force cut off (100%) frequency.\n\
\t-h  print command line options (this help), and exit.\n\
\t-l  low pass only, no re-sizing, specify frequency with -f.\n\
\t-m  fft measure, does first test runs to find best algo (takes a while).\n\
\t-r  reverse u and v (switch colors).\n\
\t-s  slope frequency (in MHz) (float), the lower the more gentle the cutoff.\n\
\t-x  new width in pixels.\n\
\t-y  new height in pixels.\n\
");
			exit(0);	
			break;
		case 'l':
			low_pass_only_flag = 1 - low_pass_only_flag;
			break;
		case 'm':
			fft_measure_flag = 1 - fft_measure_flag;
			break;
		case 'r':
			reverse_uv_flag = 1 - reverse_uv_flag;
			break;
		case 's':
			dfslope = 1000000.0 * atof(optarg);
			break;
		case 'x':
			new_width = atoi(optarg);
			break;
		case 'y':
			new_height = atoi(optarg);
			break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stderr, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr,\
				"Unknown option character `\\x%x', try -h\n.",\
				optopt);
				}
			exit(1);
			break;			
		default:
			break;
		}/* end switch a */
	}/* end while getopt() */

if(debug_flag)
	{
	fprintf(stderr,\
	"command line parser result: new_width=%d new_height=%d burst_mask=%d\n",\
	new_width, new_height, burst_mask);
	}

/* read yuv stream from stdin */
/*
From mjpegtools:
Description of the (new!, forever?) YUV4MPEG2 stream format:

STREAM consists of
 o one '\n' terminated STREAM-HEADER
 o unlimited number of FRAMEs

FRAME consists of
 o one '\n' terminated FRAME-HEADER
 o "length" octets of planar YCrCb 4:2:0 image data
   (if frame is interlaced, then the two fields are interleaved)

STREAM-HEADER consists of
 o string "YUV4MPEG2 "  (note the space after the '2')
 o unlimited number of ' ' separated TAGGED-FIELDs
 o '\n' line terminator

FRAME-HEADER consists of
 o string "FRAME "  (note the space after the 'E')
 o unlimited number of ' ' separated TAGGED-FIELDs
 o '\n' line terminator

TAGGED-FIELD consists of
 o single ascii character tag
 o VALUE (which does not contain whitespace)

VALUE consists of
 o integer (base 10 ascii representation)
 or o RATIO
 or o single ascii character
 or o generic ascii string

RATIO consists of
 o numerator (integer)
 o ':' (a colon)
 o denominator (integer)

The currently supported tags for the STREAM-HEADER:
	 W - [integer] frame width, pixels, should be > 0
 H - [integer] frame height, pixels, should be > 0
 I - [char] interlacing:  p - progressive (none)
                          t - top-field-first
                          b - bottom-field-first
                          ? - unknown
 F - [ratio] frame-rate, 0:0 == unknown
 A - [ratio] sample (pixel) aspect ratio, 0:0 == unknown
 X - [character string] 'metadata' (unparsed, but passed around)

The currently supported tags for the FRAME-HEADER:
 X - character string 'metadata' (unparsed, but passed around)
*/

/* stream stream header to get width and height and interlace */

/*
YUV4MPEG2 W720 H576 F25:1 I? A0:0 XM2AR002
FRAME
data
*/

/* create a line buffer */
bptr = malloc(READSIZE);
if(! bptr)
	{
	fprintf(stderr, "resizer-yuv(): malloc bptr failed, aborting.\n");

	exit(1);
	}


#ifndef NO_GNU_C
/*
From libc.info:
ssize_t getline (char **LINEPTR, size_t *N, FILE *STREAM)
This function is a GNU extension, but it is the recommended way to
read lines from a stream.  The alternative standard functions are 
unreliable.
*/

b = READSIZE;
a = getline(&bptr, &b, stdin);
if(a == -1)
	{
	fprintf(stderr,\
	"resizer-yuv(): EOF in read stream header, stop.\n");

	exit(0);
	}
#else
/*
code for non GNU systems contributed by "Steven M. Schultz" <sms@2BSD.COM>
*/
for(b = 0; b < READSIZE; b++)
	{
	bptr[b] = fgetc(stdin);
	if(feof(stdin))
		{
		fprintf(stderr, "resizer-yuv: EOF in read stream header, stop.\n");

		exit(0);
		}

	if (bptr[b] == '\n') break;
	}
/* keep the /n */
bptr[b + 1] = '\0';

if(b == READSIZE)
	{
	fprintf(stderr,\
	"resizer-yuv: No newline in first %d bytes of stream header\n",\
	READSIZE);

	exit(1);
	}
#endif

if(strncmp(bptr, "YUV4MPEG2", 9) != 0)
	{
	fprintf(stderr,\
	"resizer-yuv(): YUV4MPEG2 not found in stream header, aborting.\n");

	exit(1);
	}

if(debug_flag)
	{
	fprintf(stderr, "incoming stream header=%s\n", bptr);
	}

/* parse arguments */
running = strsave(bptr);
if(! running)
	{
	fprintf(stderr,\
	"resizer-yuv(): strsave(options) failed, aborting.\n");

	exit(1);
	}

while(1)
	{
	token = strsep (&running, " ");
	if(token == NULL) break;

	/* avoid empty string */
	if(token[0] == 0) continue;
	sscanf(token, "A%d:%d", &aspect_ratio1, &aspect_ratio2);
	sscanf(token, "W%d", &width);
	sscanf(token, "H%d", &height);
	sscanf(token, "F%d:%d", &frame_rate1, &frame_rate2);
	sscanf(token, "I%c", &interlace_mode);
	sscanf(token, "X%s", metadata);
	} /* end while parse tags */

free(running);

if(debug_flag)
	{
	fprintf(stderr,\
	"width=%d height=%d\n\
	frame_rate1=%d frame_rate2=%d\n\
	interlace_mode=%c\n\
	aspect_ratio=%.2f\n",
	width, height,\
	frame_rate1, frame_rate2,\
	interlace_mode,\
	aspect_ratio);
	}

if(low_pass_only_flag)
	{
	new_width = width;
	new_height = height;
	}

/* keep aspect */
new_aspect_ratio1 = aspect_ratio1;
new_aspect_ratio2 = aspect_ratio2;

/* create plans for fft and reverse fft */
if(fft_measure_flag)
	{
	fprintf(stderr,\
	"resizer-yuv(): testing fft algos for best speed, please wait.\n");

	a = FFTW_MEASURE;
	}
else a = 0;

/* creates plan for forward fft */
plan_luma = rfftw_create_plan(\
	width, FFTW_REAL_TO_COMPLEX, FFTW_OUT_OF_PLACE | a);
fprintf(stderr, "have forward luma plan.\n");

/* creates plan for forward fft color */
plan_chroma = rfftw_create_plan(\
	width / 2, FFTW_REAL_TO_COMPLEX, FFTW_OUT_OF_PLACE | a);
fprintf(stderr, "have forward chroma plan.\n");

/* create plan for reverse fft */
plan_reverse_luma = rfftw_create_plan(\
	width, FFTW_COMPLEX_TO_REAL, FFTW_OUT_OF_PLACE | a);
fprintf(stderr, "have reverse luma plan.\n");	

/* create plan for reverse fft color */
plan_reverse_chroma = rfftw_create_plan(\
	width / 2, FFTW_COMPLEX_TO_REAL, FFTW_OUT_OF_PLACE | a);
fprintf(stderr, "have reverse chroma plan.\n");

if(low_pass_only_flag)
	{
	fprintf(stderr,\
	"Found -l flag, using lowpass only mode, no resizing done!\n");
	}
else
	{
	/* setup the re-sizer */
	bb_resize_setup(width, height, new_width, new_height, debug_flag);
	}

/* caculate part of spectrum to mask out */
if(dcut_off_frequency != 0.0)
	{
	/*
	In case of PAL, active line part is about 50 us (approx) of 64 us.
	For 720 pixels in 50 us that gives 50/720 us / pixel, or 0.07 us or
	70 ns.
	70 ns corresponds to 1 / 7e-5 Hz, so 14285714 Hz or 14.2 MHz	
	However 2 pixels are required to make a period, so it is 7.1 MHz.

	fftw doc says: f = k / n, so k = f / n.

	We are going to use dh_scale_ratio, which is later already divided
	by 2, so it doubles (no pun intended) as k here.

	7100000 Hz corresponds to 720 pixels in PAL, so 7100000 with width 720
	should give a dh_scale_ratio of 1.0.
	*/

#define PAL_SYSTEM
#ifdef PAL_SYSTEM
	/* calculate time for one pixel */
	da = .00005 / width;

	/* calculate frequency */
	da = 1.0 / da;

	/*
	calculate real frequency, one period displays a white and black pixel,
	so 2 pixels per period.
	*/
	dfmax = da / 2.0;

	/* the maximum frequency */
	dfstep = dfmax / width;
#endif /* PAL_SYSTEM */

	/* where to cut in the frequency spectrum array */
	dh_scale_ratio = dcut_off_frequency / dfstep;
	}
else
	{
	/* calculate horizontal scaling ratio */
	dh_scale_ratio = (double)new_width / (double)width; 
	}

if(debug_flag)
	{
	fprintf(stderr, "dh_scale_ratio=%.2f\n", dh_scale_ratio);
	}

/* calculate length */
ylength = width * height;
ulength = (width / 2) * (height / 2);
vlength = (width / 2) * (height / 2);
length = ylength + ulength + vlength;

inbuffer = malloc(length);
if(! inbuffer)
	{
	fprintf(stderr, "resizer-yuv(): malloc inbuffer failed, aborting\n");

	exit(1);
	}

/* calculate new_length */
new_ylength = new_width * new_height;
new_ulength = (new_width / 2) * (new_height / 2);
new_vlength = (new_width / 2) * (new_height / 2);
new_length =  new_ylength + new_ulength + new_vlength;

outbuffer = malloc(new_length);
if(! outbuffer)
	{
	fprintf(stderr, "resizer-yuv(): malloc outbuffer failed, aborting\n");

	exit(1);
	}

if(debug_flag)
	{
	fprintf(stderr,\
	"main(): calculated length=%d new_length=%d\n",\
	length, new_length);

	fprintf(stderr, \
	"inbuffer=%p outbuffer=%p\n", inbuffer, outbuffer);
	}

in = malloc(width * sizeof(fftw_real) );
if(! in)
	{
	fprintf(stderr, "resizer-yuv(): malloc in[] failed, aborting\n");

	exit(1);
	}

out = malloc(width * sizeof(fftw_real) );
if(! out)
	{
	fprintf(stderr, "resizer-yuv(): malloc out[] failed, aborting\n");

	exit(1);
	}

dattenuation_factor = malloc( (width / 2) * sizeof(double) );
if(! dattenuation_factor)
	{
	fprintf(stderr,\
	"resizer-yuv(): malloc dattenuation_factor failed, aborting\n");

	exit(1);
	}

/* set pointers */
py = inbuffer;   
pyo = outbuffer;

if(! reverse_uv_flag)
	{
	pu = inbuffer + width * height;
	pv = inbuffer + (width * height * 5) / 4;

	puo = outbuffer + new_width * new_height;
	pvo = outbuffer + (new_width * new_height * 5) / 4;
	}
else
	{
	pv = inbuffer + width * height;
	pu = inbuffer + (width * height * 5) / 4;

	pvo = outbuffer + new_width * new_height;
	puo = outbuffer + (new_width * new_height * 5) / 4;
	}	

/* send new stream header */
fprintf(outfile,\
"YUV4MPEG2 W%d H%d F%d:%d I%c A%d:%d X%s\n",\
new_width, new_height, frame_rate1, frame_rate2, interlace_mode,\
new_aspect_ratio1, new_aspect_ratio2, metadata);

/*
calculate slope
Trying to make a much as possible a smooth cutof, preventing 'ringing',
this is especially visible at low cutof frequencies in black areas.

Specification of filter slope frequency:
form A to B is one half period of a sinewave of frequency dfslope,
this is what you enter with the -s  command line flag.
The -f command line flag is the frequency at which the amplitude is 1/2,
in point C.
So the filter starts a bit before that point and ends a bit after that point.
Try flags -l -f 3.5 -s 1 , this creates a low pass 50 % down at 3.5 MHz,
and rolls of smoothly to be 100 % down at 4.0 MHz.
Thats is if I did the math right.
View graph with fixed spacing!

      A                  B
. . . . 
           .
             .
              .
               .
              C .
                 .
                  . 
                   .
                      .
                           . . . .

*/

fprintf(stderr,\
"width=%d\n\
dh_scale_ratio=%.2f\n\
dfslope=%.2f dfmax=%.2f dfstep=%2.f\n\
dcut_off_frequency=%.2f\n",\
width, dh_scale_ratio, dfslope, dfmax, dfstep, dcut_off_frequency);

/* set filter slope */
points = width / 2;

/* points of a slope period */	
dslope_points = dfslope / dfstep;

/* points of a half slope period (180 degrees) */
db = dslope_points / 2.0;

/* because the fft result is in half the input point size, divide by 2 */
db /= 2.0;
fprintf(stderr, "filter slope in points=%.2f\n", db);

/* cutoff middle point */
da = dcut_off_frequency / dfstep;
	
/* because the fft result is in half the input point size, divide by 2 */
da /= 2.0;
fprintf(stderr, "cutoff point=%.2f\n", da);

/* start the slope 90 degrees earlier */
dc = da - (db / 2.0);

fprintf(stderr, "dc=%.2f\n", dc);

in_slope_flag = 0;
dangle = 90.0;
for(x = 0; x < points; x++)
	{
	/* default */
	dattenuation_factor[x] = 1.0;

	if(in_slope_flag == 0)
		{
		if(x > dc)
			{
			/* start the slope */
			in_slope_flag = 1;
			}
		}

	if(in_slope_flag == 1)
		{
		/* in this area we follow the down going part of a sine function */ 

		dattenuation_factor[x] = 1 + sin( (dangle / 360.0) * 2 * M_PI);
		dattenuation_factor[x] /= 2.0;

		dangle += 360.0 / (dslope_points / 2.0);
		if(dangle >= 270.0)
			{
			/* signal end of curve */
			in_slope_flag = 2;
			}
		}

	if(in_slope_flag == 2)
		{
		dattenuation_factor[x] = 0.0;
		}

	} /* end for all points */

//#define DEBUG_CUTOFF
#ifdef DEBUG_CUTOFF
for(i = 0; i < points; i++)
	{
	if(dattenuation_factor[i] != 1.0)
		fprintf(stderr, "a[%d]=%.2f ", i, dattenuation_factor[i]); 
	}
#endif

/* read frames in YUV format from stdin */
current_frame_nr = 1;
while(1)
	{
	/* read frame header */
#ifndef NO_GNU_C
	b = READSIZE;
	a = getline(&bptr, &b, stdin);
	if(a == -1)
		{
		fprintf(stderr,\
		"resizer-yuv(): EOF in read frame header, stop.\n");
		
		exit(0);
		}
#else
	/*
	code for non GNU systems contributed by Steven M. Schultz
	*/
	for(b = 0; b < READSIZE; b++)
		{
		bptr[b] = fgetc(stdin);
		if(feof(stdin))
			{
			fprintf(stderr,\
			"resizer-yuv: EOF in read frame header, stop.\n");

			exit(0);
			}

		if (bptr[b] == '\n') break;
		}
	/* keep the /n */
	bptr[b + 1] = '\0';

	if(b == READSIZE)
		{
		fprintf(stderr,\
		"resizer-yuv: No newline in first %d bytes of frame header, aborting.\n",\
		READSIZE);

		exit(1);
		}
#endif

	if(strncmp(bptr, "FRAME", 5) != 0)
		{
		fprintf(stderr,\
		"resizer-yuv(): No FRAME found in frame header, aborting.\n");

		exit(1);
		}

	/* 
	This is how it is written:
	Write Y', Cb, and Cr planes
	for (i = 0; i < 3; i++)
		{
		unsigned char *srctop = upper_field[i];
		unsigned char *srcbot = lower_field[i];
		// alternately write one line from each
		for (y = 0; y < height; y += 2)
			{
			if (y4m_write(fd, srctop, width)) return Y4M_ERR_SYSTEM;
			srctop += width;
			if (y4m_write(fd, srcbot, width)) return Y4M_ERR_SYSTEM;
			srcbot += width;
			}
		// for chroma, width/height are half as big
		if (i == 0)
			{
			width /= 2;
			height /= 2;
			}
		}
	*/

	/* read in frame data */
	a = fread(inbuffer, sizeof(char), length, stdin);
	if(a != length)
		{
		fprintf(stderr,\
		"resizer-yuv(): unexpected EOF in frame data, aborting.\n");

		exit(1);
		}

	if(debug_flag)
		{
		fprintf(stderr, "main(): have frame=%d\n", current_frame_nr);
		}

	/*
	prevent aliasing in the rescaler by removing all frequencies above
	f / 2 (in 720 width this makes 360).
	*/

	/* Y luminance low pass */
	points = width;
	for(y = 0; y < height; y++)
		{
		idy = y * points;

		/* get data in right format */
		for(x = 0; x < points; x++)
			{
			/* mask out burst area if specidied */
			if(x < burst_mask)
				{
				in[x] = 0.0;
				}
			else
				{
				in[x] = (double)py[idy + x];			
				}
			}


		/* forward fft */
		rfftw_one(plan_luma, in, out);

		/* modify frequency spectrum */
		for(x = 0; x < points / 2; x++)
			{
			db = (double)points * dh_scale_ratio;
			db /= 2.0;

			/*
			fft result real part in first half array,
			imaginary part in second half array.
			*/

			/* real part */
			out[x] *= dattenuation_factor[x];

			/* complex part */
			out[points - x] *= dattenuation_factor[x];

//out[points - x] = 0.0;
//out[x] = 0.0;
			} /* end for all x (frequency components) */
//out[0] *= 10.0;

//out[0] /= 100000.0;

//out[0] = 0.0;
		/* reverse fft */
		rfftw_one(plan_reverse_luma, out, in);

		/* data back to right format */
		for(x = 0; x < points; x++)
			{
			da = in[x];

			/* correct amplitude */
			da /= points;

//#define TEST_RANGE_MODE
#ifdef TEST_RANGE_MODE
			if(da > 255) fprintf(stderr, "WAS da=%.2f\n", da);
			if(da < 0) fprintf(stderr, "WAS da=%.2f\n", da);
#endif TEST_RANGE_MODE

			/* limit range, sometimes this is < 0 and sometimes > 255! */
			if(da > 255) da = 255;
			if(da < 0) da = 0;

			py[idy + x] = (unsigned char)da;
			}
		} /* end for y */

	/* U chrominance low pass */
	points = width / 2;
	for(y = 0; y < height / 2; y++)
		{
		idy = y * points;

		/* get data in right format */
		for(x = 0; x < points; x++)
			{
			/* mask out burst area if specidied */
			if(x < (burst_mask / 2) )
				{
				in[x] = 127.0;
				}
			else
				{
				in[x] = (double) pu[idy + x];
				}
			}

		/* forward fft */
		rfftw_one(plan_chroma, in, out);

		/* mask out high frequencies */
		for(x = 0; x < points / 2; x++)
			{
			/* real part */
			da = out[x] - 127;
			da *= dattenuation_factor[x * 2];
			out[x] = da + 127; 

			/* complex part */
			da = out[points - x] - 127;
			da *= dattenuation_factor[x * 2];
			out[points - x] = da + 127;

//out[points - x] = 127.0;
//out[x] = 127.0;

			}

		/* reverse fft */
		rfftw_one(plan_reverse_chroma, out, in);

		/* data back to right format */
		for(x = 0; x < points; x++)
			{
			da = in[x];

			/* correct amplitude */
			da /= points;

			/* limit range */
			if(da > 255) da = 255;
			if(da < 0) da = 0;

			pu[idy + x] = (unsigned char)da;
			}
		} /* end for y */

	/* V chrominance low pass */
	points = width / 2;
	for(y = 0; y < height / 2; y++)
		{
		idy = y * points;

		/* get data in right format */
		for(x = 0; x < points; x++)
			{
			/* mask out burst area if specidied */
			if(x < (burst_mask / 2) )
				{
				in[x] = 127.0;
				}
			else
				{
				in[x] = (double)pv[idy + x];
				}
			}

		/* forward fft */
		rfftw_one(plan_chroma, in, out);

		/* mask out high frequencies */
		for(x = 0; x < points / 2; x++)
			{
			/* real part */
			da = out[x] - 127;
			da *= dattenuation_factor[x * 2];
			out[x] = da + 127; 

			/* complex part */
			da = out[points - x] - 127;
			da *= dattenuation_factor[x * 2];
			out[points - x] = da + 127;

//out[points - x] = 127.0;
//out[x] = 127.0;
			}

		/* reverse fft */
		rfftw_one(plan_reverse_chroma, out, in);

		/* data back to right format */
		for(x = 0; x < points; x++)
			{
			da = in[x];

			/* correct amplitude */
			da /= points;

			/* limit range */
			if(da > 255) da = 255;
			if(da < 0) da = 0;

			pv[idy + x] = (unsigned char)da;
			}
		} /* end for y */

	/* data now is in inbuffer */

	if(!low_pass_only_flag)
		{
		bb_resize_frame(py, pu, pv, pyo, puo, pvo);
		}

	/* echo frame header */
	fprintf(outfile, "%s", bptr);

	/* write the frame to the output */
	if(low_pass_only_flag)
		{
		a = fwrite(inbuffer, sizeof(char), new_length, outfile);
		}
	else
		{
		a = fwrite(outbuffer, sizeof(char), new_length, outfile);
		}
	if(a != new_length)
		{
		fprintf(stderr,\
		"resizer-yuv(): fwrite(): failed,\n\
		tried to write %d bytes, only could write %d bytes, aborting.\n",\
		new_length, a);

		exit(1);
		}

	current_frame_nr++;
	} /* end while frames */

/* never here! */

/* destroy the fft and reverse fft plans (it is possible to save these) */
//rfftw_destroy_plan(plan_luma);
//rfftw_destroy_plan(plan_reverse_luma);
//rfftw_destroy_plan(plan_chroma);
//rfftw_destroy_plan(plan_reverse_chroma);

exit(1);
} /* end function main */

